package com.txznet.android_call;

import java.lang.reflect.Method;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.RemoteException;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.KeyEvent;

import com.android.internal.telephony.ITelephony;
import com.txznet.sdk.TXZCallManager;
import com.txznet.sdk.TXZCallManager.CallToolStatusListener;
import com.txznet.sdk.TXZCallManager.Contact;

public class AndroidCallTool implements TXZCallManager.CallTool {
	public static final String TAG = "AndroidCallTool";

	static TelephonyManager mTelephonyManager = (TelephonyManager) CallApp
			.getApp().getSystemService(Context.TELEPHONY_SERVICE);

	public static ITelephony getITelephony() {
		Class<?> c = TelephonyManager.class;
		Method getITelephonyMethod = null;
		try {
			// 获取声明的方法
			getITelephonyMethod = c.getDeclaredMethod("getITelephony",
					(Class[]) null);
			getITelephonyMethod.setAccessible(true);
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}

		try {
			return (ITelephony) getITelephonyMethod.invoke(mTelephonyManager,
					(Object[]) null); // 获取实例
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	private static void answerPhoneAIDL() throws RemoteException {
		ITelephony telephonyService = getITelephony();
		telephonyService.answerRingingCall();
	}

	private static void answerPhoneHeadsetHook() {
		Log.d(TAG, "answerPhoneHeadsetHook");
		Intent buttonUp = new Intent(Intent.ACTION_MEDIA_BUTTON);
		buttonUp.putExtra(Intent.EXTRA_KEY_EVENT, new KeyEvent(
				KeyEvent.ACTION_UP, KeyEvent.KEYCODE_HEADSETHOOK));
		try {
			CallApp.getApp().sendOrderedBroadcast(buttonUp,
					"android.permission.CALL_PRIVILEGED");
			Log.d(TAG, "AndroidCallTool ACTION_MEDIA_BUTTON broadcasted...");
		} catch (Exception e) {
			e.printStackTrace();
		}

		Intent headSetUnPluggedintent = new Intent(Intent.ACTION_HEADSET_PLUG);
		headSetUnPluggedintent.addFlags(Intent.FLAG_RECEIVER_REGISTERED_ONLY);
		headSetUnPluggedintent.putExtra("state", 1); // 0 = unplugged 1 =
														// Headset with
														// microphone 2 =
														// Headset without
														// microphone
		headSetUnPluggedintent.putExtra("name", "Headset");
		try {
			CallApp.getApp().sendOrderedBroadcast(headSetUnPluggedintent, null);
			Log.d(TAG, "AndroidCallTool ACTION_HEADSET_PLUG broadcasted ...");
		} catch (Exception e) {
			e.printStackTrace();
			Log.d(TAG, "AndroidCallTool Catch block of ACTION_HEADSET_PLUG broadcast");
		}

		Log.d(TAG, "AndroidCallTool Call Answered using headsethook");
	}

	private static void endCall() {
		ITelephony telephonyService = getITelephony();
		// Silence the ringer and answer the call!
		Log.d(TAG, "endCall");
		try {
			Log.d(TAG, "endCall in");
			telephonyService.endCall();
		} catch (RemoteException e) {
			e.printStackTrace();
			Log.d(TAG,
					"Error trying to endCall using telephony service.  Falling back to headset.");
			answerPhoneHeadsetHook();
		}
	}

	@Override
	public boolean acceptIncoming() {
		Log.d(TAG, "AndroidCallTool acceptIncoming");

		TelephonyManager tm = (TelephonyManager) CallApp.getApp()
				.getSystemService(Context.TELEPHONY_SERVICE);
		Log.d(TAG, "AndroidCallTool acceptIncoming tm.getCallState() : " + tm.getCallState());
		if (tm.getCallState() != TelephonyManager.CALL_STATE_RINGING) {
			return false;
		}
		try {
			answerPhoneAIDL();
		} catch (Exception e) {
			e.printStackTrace();
			Log.e(TAG,
					"Error trying to answer using telephony service.  Falling back to headset.");
			answerPhoneHeadsetHook();
		}
		return true;
	}

	@Override
	public boolean hangupCall() {
		Log.d(TAG, "AndroidCallTool hangupCall");
		endCall();
		return true;
	}

	@Override
	public boolean makeCall(Contact con) {
		Log.d(TAG, "AndroidCallTool makeCall");

		if (con == null)
			return false;
		try {
			Intent callIntent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:"
					+ con.getNumber()));
			callIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			CallApp.getApp().startActivity(callIntent);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	@Override
	public boolean rejectIncoming() {
		Log.d(TAG, "AndroidCallTool rejectIncoming");

		endCall();
		return true;
	}

	private static CallStatus mLastStatus = null;
	private static Contact mLastContact = null;
	private static CallToolStatusListener mCallToolStatusListener = null;

	public static void onIncoming(Contact con) {
		Log.d(TAG, "AndroidCallTool onIncoming");

		mLastContact = con;
		if (mCallToolStatusListener != null)
			mCallToolStatusListener.onIncoming(mLastContact, true, true);
	}

	public static void onMakeCall(Contact con) {
		Log.d(TAG, "AndroidCallTool onMakeCall");

		mLastContact = con;
		if (mCallToolStatusListener != null)
			mCallToolStatusListener.onMakeCall(mLastContact);
	}

	public static void onIdle() {
		Log.d(TAG, "AndroidCallTool onIdle");

		mLastContact = new Contact();
		if (mCallToolStatusListener != null)
			mCallToolStatusListener.onIdle();
	}

	public static void onOffhook() {
		Log.d(TAG, "AndroidCallTool onOffhook");

		if (mCallToolStatusListener != null)
			mCallToolStatusListener.onOffhook();
	}

	@Override
	public CallStatus getStatus() {
		if (mTelephonyManager != null) {
			switch (mTelephonyManager.getCallState()) {
			case TelephonyManager.CALL_STATE_IDLE: {
				if (mLastStatus != CallStatus.CALL_STATUS_IDLE) {
					onIdle();
				}
				return mLastStatus = CallStatus.CALL_STATUS_IDLE;
			}
			case TelephonyManager.CALL_STATE_OFFHOOK:
				if (mLastStatus != CallStatus.CALL_STATUS_OFFHOOK) {
					onOffhook();
				}
				return mLastStatus = CallStatus.CALL_STATUS_OFFHOOK;
			case TelephonyManager.CALL_STATE_RINGING:
				if (mLastStatus != CallStatus.CALL_STATUS_RINGING) {
					onIncoming(mLastContact);
				}
				return mLastStatus = CallStatus.CALL_STATUS_RINGING;
			}
		}
		return mLastStatus = null;
	}

	@Override
	public void setStatusListener(CallToolStatusListener listener) {
		mCallToolStatusListener = listener;
		if (mCallToolStatusListener != null) {
			// 通知电话功能可以用了
			mCallToolStatusListener.onEnabled();
		}
		// 取一次电话状态，同时设置电话的状态
		getStatus();
	}

}
